/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.ebpm.attachments.AttachmentFacade;
import org.eclipse.ebpm.attachments.AttachmentUtils;
import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.BaseSpagicService;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Status;
import org.eclipse.ebpm.messaging.core.MessageImpl;
import org.eclipse.ebpm.workflow.api.IControlAPI;
import org.eclipse.ebpm.workflow.api.IProcessEngine;
import org.eclipse.ebpm.workflow.api.Variable;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic3.components.bpm.deploy.DeploymentHelper;
import org.spagic3.exchanges.IExchangeStoreManager;


public class BPMComponent extends BaseSpagicService  {
		
	public String process = null;
	protected Logger logger = LoggerFactory.getLogger(BPMComponent.class);
	
	private final AtomicReference<IProcessEngine> processEngine = new AtomicReference<IProcessEngine>(null);
	private final AtomicReference<IExchangeStoreManager> exchangestoreManager = new AtomicReference<IExchangeStoreManager>(null);
	private EventAdmin ea = null;
	private boolean isReady = false;
	
	public IProcessEngine getProcessEngine() {
		return processEngine.get();
	}

	public void unsetProcessEngine(IProcessEngine processEngine) {
		this.processEngine.compareAndSet(processEngine, null);
	}
	
	public void setProcessEngine(IProcessEngine processEngine) {
		this.processEngine.set(processEngine);
	}

	
	public void unsetExchangeStoreManager(IExchangeStoreManager exchangeStoreManager) {
		this.exchangestoreManager.compareAndSet(exchangeStoreManager, null);
	}
	
	public void setExchangeStoreManager(IExchangeStoreManager exchangeStoreManager)  {
		this.exchangestoreManager.set(exchangeStoreManager);
	}
	
	public void bind(EventAdmin ea){
		this.ea = ea;
	}

	public void unbind(EventAdmin ea){
		this.ea = null;
	}

	public void init(){
		try{
			this.process = propertyConfigurator.getString("process");
			DeploymentHelper depHelper = new DeploymentHelper(process);
			depHelper.deploy();
			BPMContextSingleton.getInstance().register(this);
		}catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new RuntimeException(" Cannot instantiate BPM Component", e);
		}
	}
	
	
	
	public void beforeDestroy(){
		BPMContextSingleton.getInstance().unregister(this);
	}
	
    
	/**
	 * Starts a new process instance synchronously or asynchronously.
	 * The exchange parameters contains the XML that will be stored in the variable whose name is BPMContextSingleton.XML_MESSAGE.
	 * To each process should correspond an instance of this class (because there is a .service for each process).
	 */
	public void process(Exchange exchange){
	   
		if (exchange.getStatus() == Status.Active) {
			

			IControlAPI controlAPI = getProcessEngine().getControlAPI();
			
			String exchangeID = exchange.getId();
			Message in =  exchange.getIn(false);
			
			String xmlMessage =in.getBodyText();
			
			List<Variable> variables = new ArrayList<Variable>();
			
			Variable[] var = null;
 			
			Variable xmlMessageVariable = new Variable();
			xmlMessageVariable.setName(BPMContextSingleton.XML_MESSAGE);
			xmlMessageVariable.setValue(xmlMessage);
			variables.add(xmlMessageVariable);
			
			
		
			
			Variable orchestrationServiceIdVariable = new Variable();
			orchestrationServiceIdVariable.setName(BPMContextSingleton.ORCHESTRATION_SERVICE_ID);
			orchestrationServiceIdVariable.setValue(this.getSpagicId());
			variables.add(orchestrationServiceIdVariable);
			
			if (in instanceof MessageImpl ){
				String encodedAttachments = ((MessageImpl)in)._encodeAttachments();
				if ( encodedAttachments != null ) 	{
					Variable attachments = new Variable();
					attachments.setName(BPMContextSingleton.ATTACHMENTS);
					attachments.setValue(encodedAttachments);
					variables.add(attachments);
				}
			}
			
			String msgHeaders = ExchangeUtils.headerAsXML(in);
			if ( msgHeaders != null ) {
				Variable msgHeadersV = new Variable();
				msgHeadersV.setName(BPMContextSingleton.MESSAGE_HEADERS);
				msgHeadersV.setValue(msgHeaders);
				variables.add(msgHeadersV);
			}
			

			if (ExchangeUtils.isInOnly(exchange)) {
				// If exchange is InOnly means fire and forget send back the
				// done and execute the process
				done(exchange);
				var = new Variable[variables.size()];
				var = variables.toArray(var);
				long pid = controlAPI.startByProcessName(process, var);
				
				// Send notification for monitoring
				notify(exchangeID, SpagicConstants._INTERNAL_EVENT_PROCESS_STARTED, pid);
				IExchangeStoreManager lExchangeStoreManager = this.exchangestoreManager.get();
				// Store exchange for managing the process response
				lExchangeStoreManager.storeExchange(pid, exchange);
			} else if (ExchangeUtils.isInAndOut(exchange)) {
				if (ExchangeUtils.isSync(exchange)) {
					
					Variable requireSyncExecution = new Variable();
					requireSyncExecution.setName(BPMContextSingleton.REQUIRE_BPM_SYNC_EXECUTION);
					requireSyncExecution.setValue("TRUE");
					variables.add(requireSyncExecution);
					
					var = new Variable[variables.size()];
					var = variables.toArray(var);
					
					long pid = controlAPI.startByProcessName(process, var);
					
					// Send notification for monitoring
					notify(exchangeID, SpagicConstants._INTERNAL_EVENT_PROCESS_STARTED, pid);
					
					Variable varOut = controlAPI.getGlobalVariable(pid,
							BPMContextSingleton.XML_MESSAGE);
					
					Message out = exchange.getOut(true);
					out.setBody((String)varOut.getValue());
					if (out instanceof MessageImpl ){
						Variable attachmentInProcessVariable = controlAPI.getGlobalVariable(pid,
								BPMContextSingleton.ATTACHMENTS);

						if (attachmentInProcessVariable != null){
							String attachmentsInProcess = (String)attachmentInProcessVariable.getValue();
							if ( attachmentsInProcess != null){
								((MessageImpl) out)._decodeAttachments(attachmentsInProcess);
							}
						}
						
					}
				} else {
					var = new Variable[variables.size()];
					var = variables.toArray(var);
					long pid = controlAPI.startByProcessName(process, var);
					// Send notification for monitoring
					notify(exchangeID, SpagicConstants._INTERNAL_EVENT_PROCESS_STARTED, pid);
					
					IExchangeStoreManager lExchangeStoreManager = this.exchangestoreManager.get();
					// Store exchange for managing the process response
					lExchangeStoreManager.storeExchange(pid, exchange);
				}

			}

		}
	}
	
	public void notify(String id, String eventType, long processId) {
		
		Map<String, Object> internalEventProperties = new HashMap<String, Object>();
		internalEventProperties.put(SpagicConstants._IS_INTERNAL_EVENT, true);
		internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_TYPE, eventType);
		internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_REFERRING_ID, id);
		internalEventProperties.put(SpagicConstants._INTERNAL_EVENT_PROCESS_ID, processId);
		Event ev = new Event(SpagicConstants.SPAGIC_GENERIC_TOPIC, internalEventProperties);
		if (this.ea != null){
			ea.postEvent(ev);
		}
	}
	
	
	public void callBack(Long pid,  String xmlMessage, String encodedAttachments, String messageHeadersAsXML, boolean isProcessAborted, boolean avoidConnectorOutput){
		IExchangeStoreManager lExchangeStoreManager = this.exchangestoreManager.get();
		
		Exchange exchange = lExchangeStoreManager.deleteExchange(pid);
		
		if ((!isProcessAborted) && (!avoidConnectorOutput)){
			if (exchange != null){
				if (ExchangeUtils.isInOnly(exchange)){
					//
					// Sent to the output connector if this is present
					//
					if (this.target != null){
						Exchange newExchange = createInOnlyExchange();
						
						Message newIn = newExchange.getIn(true);
						
						newExchange.setProperty(SpagicConstants.CORRELATION_ID, exchange.getProperty(SpagicConstants.CORRELATION_ID));
						newIn.setBody(xmlMessage);
						if (encodedAttachments != null )
							((MessageImpl)newIn)._decodeAttachments(encodedAttachments);
						
						
						ExchangeUtils.fillHeaders(newIn, messageHeadersAsXML);
						newExchange.setIn(newIn);
						send(newExchange);
					}else{
						deleteAttachments(encodedAttachments);
					}
				}else{
					configureForResponse(exchange);
					Message out = exchange.getOut(true);
					out.setBody(xmlMessage);
					if (encodedAttachments != null )
						((MessageImpl)out)._decodeAttachments(encodedAttachments);
					
					
					ExchangeUtils.fillHeaders(out, messageHeadersAsXML);
					
					exchange.setOut(out);
					send(exchange);
				}
			}else{
				//
				// The exchange could be null because the process was started manually
				// In that case we must send an InOnly Exchange only if the component has a target
				//
				if (this.target != null){
					Exchange newExchange = createInOnlyExchange();
					Message newIn = newExchange.getIn(true);
				
					newIn.setBody(xmlMessage);
					if (encodedAttachments != null )
						((MessageImpl)newIn)._decodeAttachments(encodedAttachments);
					
					
					ExchangeUtils.fillHeaders(newIn, messageHeadersAsXML);
					newExchange.setIn(newIn);
					send(newExchange);
				}else{
					// The process has no target attachments used in the process must be deleted 
					deleteAttachments(encodedAttachments);
				}
			}
		}
	}
	
	
	public void deleteAttachments(String encodedAttachments){
		if ((encodedAttachments == null ) || (encodedAttachments.trim().length() == 0))
			return;
		
		String[] attachmentTokens = AttachmentUtils.decodeAttachmentsIds(encodedAttachments);
    	HashMap<String, String> internalAttachments = AttachmentUtils.decodeAttachmentsIds(attachmentTokens);
    	for (String idInPresistentStore : internalAttachments.values()){
    		AttachmentFacade.delete(idInPresistentStore);
    	}
	}
	

	
	
	
	
}
