/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm;

import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class BPMContextSingleton {
	/**
	 * The name of the variable that in JBPM carries the XML Messages
	 */
	public static final String XML_MESSAGE = "XML_MESSAGE";
	
	public static final String WORKFLOW_UPDATER_CLASS = "WORKFLOW_UPDATER_CLASS";
	public static final String WORKFLOW_UPDATER_VARIABLE_NAME = "WORKFLOW_UPDATER_VARIABLE_NAME";
	
	private static Logger logger = LoggerFactory.getLogger(BPMContextSingleton.class);

	/**
	 * When a JBPM Process is started by BPMComponent we set two variable
	 * Originating ConnectorID
	 */
	
	public static final String ORCHESTRATION_SERVICE_ID = "ORCHESTRATION_SERVICE_ID";
	
	public static final String TOKEN_CONTEXT = "TOKEN_CONTEXT";
	
	public static final String REQUIRE_BPM_SYNC_EXECUTION = "REQUIRE_BPM_SYNC_EXECUTION";
	public static final String ATTACHMENTS = "ATTACHMENTS";
	public static final String AVOID_CONNECTOR_OUTPUT = "AVOID_CONNECTOR_OUTPUT";
	
	public static final String MESSAGE_HEADERS = "MESSAGE_HEADERS";
	
	
	public static BPMContextSingleton instance = new BPMContextSingleton();
	
	public ConcurrentHashMap<String, BPMComponent> bpmComponents = new ConcurrentHashMap<String, BPMComponent>();
	
	public ConcurrentHashMap<String, BPMComponent> getBpmComponents() {
		return bpmComponents;
	}

	public void setBpmComponents(
			ConcurrentHashMap<String, BPMComponent> bpmComponents) {
		this.bpmComponents = bpmComponents;
	}

	public static BPMContextSingleton getInstance(){
		return instance;
	}
	
	public void register(BPMComponent bpmnComponent){
		getInstance().getBpmComponents().put(bpmnComponent.getSpagicId(), bpmnComponent);
	}
	
	public void unregister(BPMComponent bpmnComponent){
		getInstance().getBpmComponents().remove(bpmnComponent.getSpagicId());
	}
	
	public static void callBack(String orchestrationServiceId,  Long processInstanceId, String xmlMessage, String encodedAttachments, String messageHeadersAsXML, boolean isProcessTerminated, boolean avoidConnectorOutput){
		
		logger.info("BPMContextSingleton::callBack -> start");
		logger.info("BPMContextSingleton::callBack -> orchestrationServiceId ["+orchestrationServiceId+"]");
		logger.info("BPMContextSingleton::callBack -> processInstanceId ["+processInstanceId+"]");
		logger.info("BPMContextSingleton::callBack -> xmlMessage ["+xmlMessage+"]");
		logger.info("BPMContextSingleton::callBack -> encodedAttachments ["+encodedAttachments+"]");
		logger.info("BPMContextSingleton::callBack -> isProcessTerminated ["+isProcessTerminated+"]");
		logger.info("BPMContextSingleton::callBack -> avoidConnectorOutput ["+avoidConnectorOutput+"]");
		BPMComponent bpmnComponent = getInstance().getBpmComponents().get(orchestrationServiceId);
		if (bpmnComponent != null){
			bpmnComponent.callBack(processInstanceId,  xmlMessage, encodedAttachments, messageHeadersAsXML, isProcessTerminated, avoidConnectorOutput);
		}else{
			logger.error("BPMContextSingleton::callBack -> BPMComponent for OrchestrationServiceID ["+orchestrationServiceId+"] is null");
		}
		
	}
	

	public static String getOrchestrationServiceIdForProcessName(String processName){
		Collection<BPMComponent> localBpmComponents = getInstance().getBpmComponents().values();
		for (BPMComponent aBPMBpmComponent : localBpmComponents){
			if (aBPMBpmComponent.process.equalsIgnoreCase(processName))
				return aBPMBpmComponent.getSpagicId();
		}
		return null;
	}
	
}
