/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.handler;

import java.io.StringReader;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.integration.api.IExchangeProvider;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.api.Pattern;
import org.hibernate.Session;
import org.jbpm.context.exe.ContextInstance;
import org.jbpm.graph.exe.ExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.Property;
import org.spagic.metadb.model.SchemaMetadata;
import org.spagic.metadb.model.Step;
import org.spagic.workflow.api.jbpm.ProcessEngine;
import org.spagic.workflow.api.jbpm.QueryAPI;
import org.spagic.workflow.api.jbpm.TokenContext;
import org.spagic3.components.bpm.BPMContextSingleton;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.InputSource;

public class NotifyMessageExchangeProvider extends AbstractExchangeWorker {

	protected Logger logger = LoggerFactory.getLogger(NotifyMessageExchangeProvider.class);
	
	
	public Exchange createExchangeFromWorkflowContext(Object workflowContext) {
		ExecutionContext context = (ExecutionContext)workflowContext;
		ContextInstance contextInstance = context.getContextInstance();
		
		Exchange exchange = ExchangeUtils.createExchange(Pattern.InOut);		
		
		TokenContext tokenContex = TokenContext.fromExecutionContext(context);
		exchange.setProperty(BPMContextSingleton.TOKEN_CONTEXT, tokenContex.toString());
		setSyncExecutionIfNeeded(context, exchange);
		
		try {
			/**
			 * 
			 * 
			 * <processinfo>
				    <processname></processname>
				    <taskname></taskname>
				    <processinstanceID></processinstanceID>
				    <variables>
				        <entry>
				            <key>nome variabile processo</key>
				            <value>valore variabile processo</value>
				        </entry>
				        <entry>
				            <key>nome variabile 2 </key>
				            <value>valore variabile 2</value>
				        </entry>
				    </variables>
				    <metadatadefinition id="id-1">
				        <field id="var1" name="var1" type="String" defaultValue="" mandatory="true" validator="" length="0" precision="0" combo="false"/>
				        <field id="var2" name="var2" type="String" defaultValue="ggg" mandatory="true" validator="None" length="0" precision="0" combo="false"/>
				        <field id="var3" name="var3" type="Long" defaultValue="" mandatory="false" validator="" length="0" precision="0" combo="false"/>
				        <column id="colonna-1" name="colonna-1" type="Date" defaultValue="01/01/2010" editable="true" mandatory="true" validator="None" length="0" precision="0" combo="false"/>
				        <column id="combo-column" name="combo-column" type="String" defaultValue="AAA" editable="true" mandatory="false" validator="Text" length="0" precision="0" combo="true">
				            <item name="AAA Name" value="AAA"/>
				            <item name="BBB Name" value="BBB"/>
				        </column>
				        <field id="var4" name="var4" type="String" defaultValue="wwwwwwwwwww" mandatory="false" validator="" length="0" precision="0" combo="true">
				            <item name="Combo nome 1" value="Combovalue1"/>
				            <item name="Combo nome 2" value="value2"/>
				        </field>
				    </metadatadefinition>
				</processinfo> 
			*
			*
			*/
			
			
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder b = factory.newDocumentBuilder();
			Document doc = b.newDocument();
			
			
	        DocumentFragment fragment = doc.createDocumentFragment();
	        Element root = doc.createElement("processinfo");
	        fragment.appendChild(root);
	        doc.appendChild(fragment);
			
	        Element processNameNode = doc.createElement("processname");
	        Text textNode = doc.createTextNode(context.getProcessDefinition().getName());
	        processNameNode.appendChild(textNode);
	        root.appendChild(processNameNode);
	        
	        
	        Element taskNameNode = doc.createElement("taskname");
	        textNode = doc.createTextNode(context.getNode().getName());
	        taskNameNode.appendChild(textNode);
	        root.appendChild(taskNameNode);
	        
	        
	        Element processinstanceIDNode = doc.createElement("processinstanceID");
	        textNode = doc.createTextNode(getProcessInstanceId(contextInstance));
	        processinstanceIDNode.appendChild(textNode);
	        root.appendChild(processinstanceIDNode);
	        
	        
	        // Produce the node with all variables
	        Element variablesNode = doc.createElement("variables");
	        root.appendChild(variablesNode);

	        Set<Map.Entry<String,Object>> set = contextInstance.getVariables().entrySet();
	        for (Map.Entry<String,Object> entry : set) {
	        	Element variableNode = makeVariableNode(doc, entry.getKey(), entry.getValue().toString());
	        	variablesNode.appendChild(variableNode);
	        }

			Session aSession = null;
	        try {
				aSession = getSession();
		        
		        // Retrieve the metadata id, to read it from the database	
				String correlationID = ProcessEngine.createSpagicCorrelationId(context.getProcessInstance());
				ProcessInstance spagicProcessInstance = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, correlationID);
				if (spagicProcessInstance == null) {
					logger.error("Unable to find ProcessInstance with correlationID: " + correlationID);
				} else {
			        Process spagicProcess = spagicProcessInstance.getProcess();
					Step step = getSpagicStepFromJBPMNode(spagicProcess, context.getNode().getName(), aSession);
			        
					Property propertyFormId = AuditDBUtils.getStepPropertyByName(aSession, step, QueryAPI.PROPERTY_FORM_ID);
					if (propertyFormId != null) {
						String schemaIdentifier = propertyFormId.getValue();
						
						if (schemaIdentifier != null) {
							// Retrieve the metadata and insert it into the notification
							SchemaMetadata md = AuditDBUtils.getSchemaDefinition(aSession, schemaIdentifier);				
							
							// Parse the XML as a W3C document.
							DocumentBuilderFactory xmlFact = DocumentBuilderFactory.newInstance();
							DocumentBuilder builder = xmlFact.newDocumentBuilder();
							InputSource source = new InputSource(new StringReader(md.getDataSchema()));
					        Document schemaDocument = builder.parse(source);
					        
					        Node importedNode = doc.importNode(schemaDocument.getDocumentElement(), true);
					        root.appendChild(importedNode);												
						}
					}
				}					
	        } finally {
				aSession.close();
	        }
	        
	        // Render the notification as an XML string (actually it's a DOM)	        
			java.io.StringWriter xmlMessage = new java.io.StringWriter();
			com.sun.org.apache.xml.internal.serialize.XMLSerializer serializer = new com.sun.org.apache.xml.internal.serialize.XMLSerializer(
					xmlMessage, new com.sun.org.apache.xml.internal.serialize.OutputFormat(doc));
			serializer.serialize(doc);
			
			
			Message in = exchange.getIn(true);
			in.setBody(xmlMessage.toString());

		} catch (Exception ex) {
			logger.error("Problem during notification of task availability", ex);
		}		
		
		return exchange;
	}

	public static Step getSpagicStepFromJBPMNode(Process spagicProcess, String nodeName, Session aSession) {
		String processName = spagicProcess.getName();
		String processNamespace = spagicProcess.getNamespace();
		String stepName = ProcessEngine.getStepLocalNamePart(processName, spagicProcess.getProcessVersion(), nodeName);
		Step targetStep = AuditDBUtils.getStepByName(aSession, processNamespace, stepName);
		return targetStep;
	}
	
	protected Session getSession() {
		Session aSession = HibernateUtil.getSessionFactory().openSession();
		return aSession;
	}

	private Element makeVariableNode(Document doc, String key, String value) {
		Element keyNode = doc.createElement("key");
		Text nametextNode = doc.createTextNode(key);
		keyNode.appendChild(nametextNode);

		Element valueNode = doc.createElement("value");
		Text phonetextNode = doc.createTextNode(value);
		valueNode.appendChild(phonetextNode);

		Element entryNode = doc.createElement("entry");
		entryNode.appendChild(keyNode);
		entryNode.appendChild(valueNode);
		return (entryNode);
	}	

	protected void setSyncExecutionIfNeeded(ExecutionContext workflowContext, Exchange exchange){
		String requireSyncExecutionString = (String)workflowContext.getVariable(BPMContextSingleton.REQUIRE_BPM_SYNC_EXECUTION);
		if ((requireSyncExecutionString != null) && (requireSyncExecutionString.equalsIgnoreCase("TRUE"))){
			exchange.setProperty(SpagicConstants.SYNC_EXCHANGE, "true");
		}
	}
	
	
	
	public String getProcessInstanceId(ContextInstance contextInstance) {
		Object var = contextInstance.getVariable(SpagicConstants.WF_PROCESS_INSTANCE_ID_VARIABLE);
		return var.toString();
	}
	

	
}
