/**

    Copyright 2007, 2010 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic3.components.bpm.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.eclipse.ebpm.constants.SpagicConstants;
import org.eclipse.ebpm.core.ExchangeUtils;
import org.eclipse.ebpm.core.ProcessContext;
import org.eclipse.ebpm.integration.api.IWorkflowContextUpdater;
import org.eclipse.ebpm.messaging.api.Exchange;
import org.eclipse.ebpm.messaging.api.Message;
import org.eclipse.ebpm.messaging.core.MessageImpl;
import org.eclipse.ebpm.workflow.api.Variable;
import org.jbpm.graph.exe.ExecutionContext;
import org.spagic.workflow.api.jbpm.ProcessEngine;
import org.spagic.workflow.logging.log.FaultLog;
import org.spagic3.components.bpm.BPMContextSingleton;

public class SetVariableAndContinueProcessUpdater extends AbstractExchangeWorker implements IWorkflowContextUpdater {


	@Override
	public void updateWorkflowContext(Object workflowContex, Exchange exchange) {
		if (workflowContex == null)
			prepareVariablesAndSignal(exchange);
		else
			updateWorkflowContextAndSignal(workflowContex,exchange);
	}
	
	public Exception getExceptionFromExchange(Exchange exchange){
		Message fault = exchange.getFault(false);
		Exception e = null;
		if (fault != null){
			e = new Exception(fault.getBodyText());
		}if (exchange.getError() != null){
			e = exchange.getError();
		}
		return e;
	}
	public FaultLog createFaultLog(ExecutionContext context, Exchange exchange){
		FaultLog faultLog = null;
		
		Exception exchangeException = getExceptionFromExchange(exchange);
		if (exchangeException != null){
			faultLog = new FaultLog();
			faultLog.setToken(context.getToken());
			faultLog.setAction(context.getAction());
			faultLog.setException(exchangeException);
			faultLog.setProcessId(context.getProcessInstance().getId());
			faultLog.setProcessName(context.getProcessDefinition().getName());
			faultLog.setNodeName(context.getNode().getName());
		}
		return faultLog;
	}
	
	public void updateWorkflowContextAndSignal(Object workflowContex,Exchange exchange) {
		ExecutionContext context = (ExecutionContext)workflowContex;
		FaultLog aFaultLog = createFaultLog(context, exchange);
	
		if (aFaultLog != null){		
			context.getProcessInstance().getRootToken().addLog(aFaultLog);
			context.getProcessInstance().end();
		}else{
			org.jbpm.graph.exe.ProcessInstance pi = context.getToken().getProcessInstance();
		
			Variable[] vars = extractVariablesFromExchange(exchange);
			for (int i = 0; (vars != null && i < vars.length); i++) {
				pi.getContextInstance().setVariable(vars[i].getName(), vars[i].getValue(), context.getToken());
			}
			context.getJbpmContext().save(pi);
			context.getToken().signal();
		}
	}
	
	public Variable[] extractVariablesFromExchange(Exchange exchange) {
		List<Variable> vars = new ArrayList<Variable>();
		
		
		String noUpdateToXMLMessage = (String) exchange.getProperty(SpagicConstants.WF_NO_UPDATE_XML_MESSAGE);
		
		boolean noUpdate = noUpdateToXMLMessage != null ? Boolean.valueOf(noUpdateToXMLMessage) : false;
		
		Message out = exchange.getOut(false);
		if (!noUpdate){
			Variable var = new Variable();
			var.setName(getVariableName());
			
			String responseXMLMessage = out.getBodyText();
			var.setValue(responseXMLMessage);
			vars.add(var);
		}else{
			Variable var = new Variable();
			var.setName(SpagicConstants.WF_NO_UPDATE_XML_MESSAGE);
			var.setValue(noUpdateToXMLMessage);
			vars.add(var);
		}
		
		String outAttachments = ((MessageImpl)out)._encodeAttachments();
		if (outAttachments != null){
			Variable varAttachments = new Variable();
			varAttachments.setName(BPMContextSingleton.ATTACHMENTS);
			varAttachments.setValue(outAttachments);
			vars.add(varAttachments);
		}else {
			Variable varAttachments = new Variable();
			varAttachments.setName(BPMContextSingleton.ATTACHMENTS);
			varAttachments.setValue("");
			vars.add(varAttachments);
		}
		
		String outHeaders = ExchangeUtils.headerAsXML(out);
		if (outHeaders != null){
			Variable headersV = new Variable();
			headersV.setName(BPMContextSingleton.MESSAGE_HEADERS);
			headersV.setValue(outHeaders);
			vars.add(headersV);
		}else{
				Variable headersV = new Variable();
				headersV.setName(BPMContextSingleton.MESSAGE_HEADERS);
				headersV.setValue("");
				vars.add(headersV);
		
		}
		vars.addAll(getProcessVariablesFromExchange(exchange));
		
		Variable[] varArray = new Variable[vars.size()];
		varArray = vars.toArray(varArray);
		return varArray;
	}
	
	
	public void prepareVariablesAndSignal(Exchange exchange) {
			
			Exception e = getExceptionFromExchange(exchange);
			String tokenContext =(String) exchange.getProperty(BPMContextSingleton.TOKEN_CONTEXT);
			if (e != null)
				ProcessEngine.endWithException(tokenContext, e);
			else
				ProcessEngine.signalToken(tokenContext, extractVariablesFromExchange(exchange));
	}

	
	
	
	public List<Variable> getProcessVariablesFromExchange(Exchange exchange){
		ProcessContext pc = new ProcessContext(exchange);
		Map<String,Object> varsMap = pc.getVariables();
		List<Variable> varList = new ArrayList<Variable>();
		
		for (String key : varsMap.keySet()){
			Variable v = new Variable();
			v.setName(key);
			v.setValue(varsMap.get(key));
			varList.add(v);
		}
		return varList;
	}

	@Override
	public void setProperties(Map<String, String> properties) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public Map<String, String> getProperties() {
		// TODO Auto-generated method stub
		return null;
	}

	

}
